//
//  InputViewControllerTests.swift
//  ToDo
//
//  Created by dasdom on 30.10.15.
//  Copyright © 2015 Dominik Hauser. All rights reserved.
//

import XCTest
@testable import ToDo
import CoreLocation

class InputViewControllerTests: XCTestCase {
    
    var sut: InputViewController!
    var placemark: MockPlacemark!
    
    override func setUp() {
        super.setUp()
        
        let storyboard = UIStoryboard(name: "Main",
            bundle: nil)
        sut = storyboard.instantiateViewControllerWithIdentifier(
            "InputViewController") as! InputViewController
        
        _ = sut.view
    }
    
    override func tearDown() {
        super.tearDown()
    }
    
    func test_HasTitleTextField() {
        XCTAssertNotNil(sut.titleTextField)
    }
    
    func test_HasDateTextField() {
        XCTAssertNotNil(sut.dateTextField)
    }
    
    func test_HasLocationTextField() {
        XCTAssertNotNil(sut.locationTextField)
    }
    
    func test_HasAddressTextField() {
        XCTAssertNotNil(sut.addressTextField)
    }
    
    func test_HasDescriptionTextField() {
        XCTAssertNotNil(sut.descriptionTextField)
    }
    
    func test_HasSaveButton() {
        XCTAssertNotNil(sut.saveButton)
    }
    
    func test_HasCancelButton() {
        XCTAssertNotNil(sut.cancelButton)
    }

    func testSave_UsesGeocoderToGetCoordinateFromAddress() {
        let mockInputViewController = MockInputViewController()
        
        mockInputViewController.titleTextField = UITextField()
        mockInputViewController.dateTextField = UITextField()
        mockInputViewController.locationTextField = UITextField()
        mockInputViewController.addressTextField = UITextField()
        mockInputViewController.descriptionTextField = UITextField()
        
        mockInputViewController.titleTextField.text = "Test Title"
        mockInputViewController.dateTextField.text = "02/22/2016"
        mockInputViewController.locationTextField.text = "Office"
        mockInputViewController.addressTextField.text = "Infinite Loop 1, Cupertino"
        mockInputViewController.descriptionTextField.text = "Test Description"
        
        let mockGeocoder = MockGeocoder()
        mockInputViewController.geocoder = mockGeocoder
        
        mockInputViewController.itemManager = ItemManager()
        
        let expectation = expectationWithDescription("bla")
        
        mockInputViewController.completionHandler = {
            expectation.fulfill()
        }
        
        mockInputViewController.save()
        
        placemark = MockPlacemark()
        let coordinate = CLLocationCoordinate2DMake(37.3316851, -122.0300674)
        placemark.mockCoordinate = coordinate
        mockGeocoder.completionHandler?([placemark], nil)
        
        waitForExpectationsWithTimeout(1, handler: nil)
        
        let item = mockInputViewController.itemManager?.itemAtIndex(0)
        
        let testItem = ToDoItem(title: "Test Title",
            itemDescription: "Test Description",
            timestamp: 1456095600,
            location: Location(name: "Office", coordinate: coordinate))
        
        XCTAssertEqual(item, testItem)
    }
    
    func testSave_CreatesToDoItemWithTitleDateAndDescription() {
        sut.titleTextField.text = "Test Title"
        sut.dateTextField.text = "02/22/2016"
        sut.descriptionTextField.text = "Test Description"

        sut.itemManager = ItemManager()
        
        sut.save()
        
        let item = sut.itemManager?.itemAtIndex(0)
        
        let testItem = ToDoItem(title: "Test Title",
            itemDescription: "Test Description",
            timestamp: 1456095600,
            location: nil)
        
        XCTAssertEqual(item, testItem)
    }
    
    func testSave_CreatesToDoItemWithTitle() {
        sut.titleTextField.text = "Test Title"
        
        sut.itemManager = ItemManager()
        
        sut.save()
        
        let item = sut.itemManager?.itemAtIndex(0)
        
        let testItem = ToDoItem(title: "Test Title")
        
        XCTAssertEqual(item, testItem)
    }
    
    func test_SaveButtonHasSaveAction() {
        let button: UIButton = sut.saveButton
        
        guard let actions = button.actionsForTarget(sut,
            forControlEvent: .TouchUpInside) else {
                XCTFail(); return
        }
        
        XCTAssertTrue(actions.contains("save"))
    }

    func test_GeocoderWorksAsExpected() {
        let expectation = expectationWithDescription("Wait for geocoder")
        
        CLGeocoder().geocodeAddressString("Infinite Loop 1, Cupertino") {
            (placemarks, error) -> Void in
            let placemark = placemarks?.first
            
            let coordinate = placemark?.location?.coordinate
            guard let latitude = coordinate?.latitude else {
                XCTFail()
                return
            }
            
            guard let longitude = coordinate?.longitude else {
                XCTFail()
                return
            }
            
            XCTAssertEqualWithAccuracy(latitude, 37.3316851,
                accuracy: 0.000001)
            XCTAssertEqualWithAccuracy(longitude, -122.0300674,
                accuracy: 0.000001)

            expectation.fulfill()
        }
        
        waitForExpectationsWithTimeout(3, handler: nil)
    }
    
    func testSave_DismissesViewController() {
        let mockInputViewController = MockInputViewController()
        
        mockInputViewController.titleTextField = UITextField()
        mockInputViewController.dateTextField = UITextField()
        mockInputViewController.locationTextField = UITextField()
        mockInputViewController.addressTextField = UITextField()
        mockInputViewController.descriptionTextField = UITextField()
        
        mockInputViewController.titleTextField.text = "Test Title"
        mockInputViewController.save()
        
        XCTAssertTrue(mockInputViewController.dismissGotCalled)
    }
}

extension InputViewControllerTests {
    class MockGeocoder: CLGeocoder {
        
        var completionHandler: CLGeocodeCompletionHandler?
        
        override func geocodeAddressString(addressString: String,
            completionHandler: CLGeocodeCompletionHandler) {
                
                self.completionHandler = completionHandler
        }
    }
    
    class MockPlacemark : CLPlacemark {
        
        var mockCoordinate: CLLocationCoordinate2D?
        
        override var location: CLLocation? {
            guard let coordinate = mockCoordinate else
            { return CLLocation() }
            
            return CLLocation(latitude: coordinate.latitude,
                longitude: coordinate.longitude)
        }
    }
    
    class MockInputViewController : InputViewController {
        
        var dismissGotCalled = false
        var completionHandler: (() -> Void)?
        
        override func dismissViewControllerAnimated(flag: Bool,
            completion: (() -> Void)?) {
                
                dismissGotCalled = true
                completionHandler?()
        }
    }
}

